#include <types.h>
#include <lib.h>
#include <Devices/Storage/atapio.h>

#include "ext2.h"

static superblock_t superblock;
static uint32_t ext2_base;
static block_group_descriptor_t group_descriptors[32];
static inode_t inode_array[4];

uint32_t ext2_init(uint32_t base)
{
	ext2_base = base;
	ata_read_sectors(ATA_MASTER, ext2_base+2, 1, superblock.data16);
	ata_read_sectors(ATA_MASTER, ext2_base+4, 2, group_descriptors->data16);

	return 0;
}

uint32_t ext2_get_inode(uint32_t inode_num, inode_t* buf)
{
	if (!buf)
		return -1;

	uint32_t group = inode_num / superblock.inodes_per_group;
	uint32_t index = inode_num % superblock.inodes_per_group -1;
	uint32_t offset = index / 4;
	uint32_t sub_index = index % 4;

	ata_read_sectors(ATA_MASTER, ext2_base + group_descriptors[group].inode_table_block_addr * 2 + offset, 1, inode_array[0].data16);
	*buf = inode_array[sub_index];


	return 0;
}

uint32_t ext2_read_sectors(uint32_t inode_num, uint32_t start, uint32_t count, uint16_t* buf)
{
	if (!buf)
		return -1;

	inode_t inode;
	ext2_get_inode(inode_num, &inode);

	uint32_t sectors_read = 0;

	// step 1: determine which block.
	uint32_t block = start / 2;

	// step 2: which sector in that block do we want?
	uint32_t sector = start % 2;
	printf("start %d block %d sector %d\n", start, block, sector);

	// read the first (maybe misaligned) sector(s)
	int i;
	for (i = sector; i < 2 && sectors_read < count; i++) {
		if (!inode.block_pointer[block])
			break;
		printf("Reading from inode %x block %x sector %x into %#x\n", inode_num, block, i, buf + sectors_read * 256);
		ata_read_sectors(ATA_MASTER, ext2_base + inode.block_pointer[block] * 2 + i, 1, buf + sectors_read * 256);
		sectors_read++;
	}

	return 0;
}

uint32_t ext2_read_data(uint32_t inode_num, uint32_t count, uint16_t* buf)
{
	if (!buf)
		return -1;

	inode_t inode;
	ext2_get_inode(inode_num, &inode);

	uint32_t blocks_left = count;
	// pass in 0 to load the whole file
	if (blocks_left == 0)
		blocks_left = (inode.sectors_used + 1) / 2;

	int i;
	for (i = 0; i < 12 && blocks_left != 0; i++) {
		if (!inode.block_pointer[i])
			break;
		ata_read_sectors(ATA_MASTER, ext2_base + inode.block_pointer[i] * 2, 2, buf + i * 512);
		blocks_left--;
	}

	// do indirect blocks later

	return 0;
}

uint32_t ext2_find_inode_num(const int8_t* filename)
{
	uint32_t inode_num = 0;

	inode_t root;
	ext2_get_inode(2, &root);

	union {
		uint16_t data16[256 * root.sectors_used];
		uint8_t data8[512 * root.sectors_used];
	} data;

	ext2_read_data(2, root.sectors_used/2, data.data16);

	uint32_t bytes_read = 0;

	dir_entry_t* entry = (dir_entry_t*)data.data16;
	while (bytes_read < root.sectors_used * 512){
		if (strncmp((int8_t*)entry->name, filename, entry->length) == 0)
			inode_num = entry->inode;
		bytes_read += entry->size;
		entry = (dir_entry_t*)((uint32_t)entry + entry->size);
	}

	return inode_num;
}

uint32_t ext2_find_inode_num2(const int8_t* filename, uint32_t parent_inode_num)
{
	uint32_t inode_num = 0;

	inode_t parent;

	if (!parent_inode_num) {
		printf("No parent provied, using root directory.\n");
		ext2_get_inode(2, &parent);
	} else {
		ext2_get_inode(parent_inode_num, &parent);
	}

	//printf("Retreived inode %x\n", parent_inode_num);

	union {
		uint16_t data16[256 * parent.sectors_used];
		uint8_t data8[512 * parent.sectors_used];
	} data;

	ext2_read_data(parent_inode_num, parent.sectors_used/2, data.data16);

	//printf("Read data from parent\n");

	uint32_t bytes_read = 0;

	dir_entry_t* entry = (dir_entry_t*)data.data16;
	while (bytes_read < parent.sectors_used * 512){
		bytes_read += entry->size;
		//printf("directory entry %s at inode %x\n", entry->name, entry->inode);
		if (strncmp((int8_t*)entry->name, filename, entry->length) == 0) {
			//printf("Found directory entry %s at inode %x\n", filename, entry->inode);
			inode_num = entry->inode;
			break;
		}
		entry = (dir_entry_t*)((uint32_t)entry + entry->size);
	}

	return inode_num;
}

uint32_t ext2_find_inode(const int8_t* filename, inode_t* inode)
{
	uint32_t inode_num = ext2_find_inode_num(filename);
	return ext2_get_inode(inode_num, inode);
}

uint32_t ext2_parse_path_to_inode_num(int8_t* filename)
{
	/* what if we just parsed the the filename and recursively
	 * called the function but just copying what was between the
	 * slashes */

	if (*filename != '/') {
		printf("Please start paths with \'/\'\n");
		return -1;
	}

	int8_t _tmpfilename[128];
	int8_t* tmpfilename = _tmpfilename;	//kind of a hack

	strcpy(filename, tmpfilename);

	// ignore the first slash, since we know its there
	tmpfilename++;
	int8_t* end_pointer = tmpfilename;

	// just a hack to return the root file node
	if (!*tmpfilename) {
		return 2;
	}

	int more_to_do = 1;

	uint32_t inode = 2;

	while(more_to_do) {
		// find the next slash
		while (*end_pointer != '/') {
			//printf("%c ", *end_pointer);
			end_pointer++;
			if (*end_pointer == NULL) {
				more_to_do = 0;
				break;
			}
		}

		// so now filename to end pointer is a terminated c string
		*end_pointer = NULL;

		//printf("\n%s\n", filename);

		// this will now find the first entry in the root directory
		
		inode = ext2_find_inode_num2(tmpfilename, inode);

		if (!inode) {
			printf("File not found: %s\n", tmpfilename);
			return inode;
		}

		// so now we got the inode of the first folder.

		// this probably isn't neccesary, but at the end of the string
		// these would both be put into other memory by 1 byte, so
		// might as well stop that even if they are out of scope soon
		if (more_to_do) {
			end_pointer++;
			tmpfilename = end_pointer;
		}

	}
	
	return inode;
}