#include <types.h>
#include <lib.h>
#include <x86_desc.h>
#include <paging.h>
#include <Filesystem/ext2.h>
#include <Devices/terminal.h>
#include <heap.h>

#include "execute.h"

#define PCB_MASK	0xffffe000

#define divhi(x, y) (x/y + (x % y != 0))

// TEMPORARY!!!
static uint32_t pid = 0;

static fops_t stdin_ops = {terminal_open, terminal_close, terminal_read, NULL};
static fops_t stdout_ops = {terminal_open, terminal_close, NULL, terminal_write};


void get_ret_addr()
{
	uint32_t ret_eip;

	// get the return address from the stack. 
	asm volatile ("         \n\
	mov 20(%%esp), %0"
	: "=r"(ret_eip));

	// add 1 so it skips over the iret;
	ret_eip++;

	// don't pass in pcb pointers cause this should be easy to call from asm
	pcb_t* pcb = (pcb_t*)(_8MB - _8KB - _8KB*pid);
	pcb->execute_return = ret_eip;
}

pcb_t* get_pcb_ptr()
{
	pcb_t* pcb;

	/* get the PCB pointer passed from assembly */
	asm volatile("              \n\
		andl %%esp, %%eax       \n\
		movl %%eax, %0"
		: "=r"(pcb)
		: "a"(PCB_MASK)
		);

	return pcb;
}


int32_t _sys_exec(uint8_t* cmd)
{

	union {
		uint16_t data16[512];
		uint8_t data8[1024];
	} data;

	// ignore leading white space
	while (*cmd == ' ') {
		*cmd = NULL;
		cmd++;
	}

	uint32_t cmdlen = strlen((int8_t*)cmd);

	// stop at the first space after that
	int len = 0;
	while (*(cmd + len) != ' ' && *(cmd + len) != NULL) {
		len++;
	}

	uint8_t name[len+1];
	strncpy((int8_t*)cmd, (int8_t*)name, len);
	name[len] = NULL;
	uint8_t* tmpargs = cmd;
	uint32_t argc = 1;

	while (*tmpargs != NULL) {
		if (*tmpargs != ' ' && *(tmpargs-1) == ' ') {
			argc++;
		}
		tmpargs++;
	}

	// This will not work for arguments which need spaces
	int i;
	for (i = 0; i < cmdlen; i++) {
		if (cmd[i] == ' ')
			cmd[i] = NULL;
	}

	uint32_t parsed_inode = ext2_parse_path_to_inode_num((int8_t*)name);

	if (parsed_inode == -1)
		return -EXEC_NOT_FOUND;

	ext2_read_data(parsed_inode, 1, data.data16);

	if (data.data8[EI_MAG0] != ELFMAG0) return EXEC_NOT_ELF;
	if (data.data8[EI_MAG1] != ELFMAG1) return EXEC_NOT_ELF;
	if (data.data8[EI_MAG2] != ELFMAG2) return EXEC_NOT_ELF;
	if (data.data8[EI_MAG3] != ELFMAG3) return EXEC_NOT_ELF;

	/* populate tss */
	tss.ss0 = KERNEL_DS;
	tss.esp0 = _8MB - _8KB*pid - 4;

	pcb_t* pcb = (pcb_t*)(_8MB - _8KB - _8KB*pid);
	pcb->pid = pid;
	pcb->parent_pid = get_pcb_ptr()->pid;

	pcb->file_desc_array[0].file_ops = &stdin_ops;
	pcb->file_desc_array[1].file_ops = &stdout_ops;

	heap_create(pid);

	/* increment pid and run process; decrement it after termination */

	uint32_t status;

	// we need to allocate space for each of the pointers, then 
	// space for each of the arguments
	uint8_t** argv = heap_alloc(sizeof(uint8_t*) * argc + cmdlen);
	uint8_t* cmdcpy = (uint8_t*)((uint32_t)argv + sizeof(uint8_t*) * argc);

	for (i = 0; i < cmdlen; i++) {
		cmdcpy[i] = cmd[i];
	}

	tmpargs = cmdcpy;
	for (i = 0; i < argc; i++) {
		while (*tmpargs == NULL) {
			tmpargs++;
			if (tmpargs - cmdcpy > cmdlen)
				break;
		}
		argv[i] = tmpargs;
		while (*tmpargs) {
			tmpargs++;
			if (tmpargs - cmdcpy > cmdlen)
				break;
		}
	}

	elf_header_t* e_header = (elf_header_t*)data.data8;


	program_header_t* p_header[e_header->e_phnum]; 
	
	for (i = 0; i < e_header->e_phnum; i++) {
		p_header[i] = (program_header_t*)(data.data8 + e_header->e_phoff + i * e_header->e_phentsize);
	}


	map_large(EXEC_BASE_ADDR, _8MB + pid * _4MB);		//where program will be run
	map_large(EXEC_TMP_ADDR, _8MB + (pid + 1) * _4MB);	// program loaded temporarily here, then copied over
	ext2_read_data(parsed_inode, 0, (uint16_t*)EXEC_TMP_ADDR);

	// what if you...
	// read the program header to see how many bytes you needed to allocate and where
	// wrote an ext2 read where you could specifiy a starting point in the inode and bytes to read

	pcb->program_break = NULL;

	for (i = 0; i < e_header->e_phnum; i++) {
		memccpy((void*)p_header[i]->p_vaddr, (const void*)(EXEC_TMP_ADDR + p_header[i]->p_offset), p_header[i]->p_memsz);
		if (p_header[i]->p_vaddr + p_header[i]->p_memsz > pcb->program_break)
			pcb->program_break = p_header[i]->p_vaddr + p_header[i]->p_memsz;
	}

	pcb->program_size = pcb->program_break;

	unmap_large(EXEC_TMP_ADDR);

	//ext2_read_sectors(parsed_inode, p_header[1]->p_offset/512, divhi(p_header[1]->p_memsz, 512), p_header[1]->p_vaddr);
	

	*USER_ARGC = argc;
	*USER_ARGV = argv;

	pid++;

	/* push use data segment, user stack pointer,
	* flags, user code segment, entry point, then iret */
	asm volatile("           \n\
	movl %%ebp, %0       \n\
	movl %%esp, %1       \n\
	pushl $0x2B          \n\
	pushl $0x83ffff8     \n\
	pushfl               \n\
	pop %%eax            \n\
	orl $0x200, %%eax    \n\
	push %%eax           \n\
	pushl $0x23          \n\
	pushl %3             \n\
	call get_ret_addr    \n\
	iret                 \n\
	pop %2"
	: "=m"(pcb->parent_ebp), "=m"(pcb->parent_esp), "=m"(status)
	: "r"(e_header->e_entry)
	: "%eax"
	);

	pid--;


	//printf("end of _sys_exec\n");
	return status;
}

int32_t _sys_exit(uint32_t status)
{
	//printf("_sys_exit\n");

	pcb_t* pcb = get_pcb_ptr();

	/* restart the shell if the user quits the last layer */
	if (!pcb->execute_return) {
		if (pcb->pid == 0) {
			pid = 0;            // this feels like a hack...
			_sys_exec((uint8_t*)"/Programs/bin/exectest");
		}
	}

	map_large(EXEC_BASE_ADDR, (_8MB + (get_pcb_ptr()->parent_pid)*_4MB));

	tss.esp0 = pcb->parent_esp;

	/* give the control back */
	asm volatile("              \
	mov %0, %%ebp           \n\
	mov %1, %%esp           \n\
	push %%ebx              \n\
	jmp *%%eax"
	: 
	:"m"(pcb->parent_ebp), "m"(pcb->parent_esp), "b"(status), "a"(pcb->execute_return)
	);

	return -1;
}
