#include "multiboot.h"
#include "lib.h"
#include "x86_desc.h"

#include "Interrupts/interrupts.h"
#include "Interrupts/exceptions.h"
#include "Interrupts/i8259.h"
#include "Devices/keyboard.h"
#include "Devices/Storage/atapio.h"
#include "Filesystem/mbr.h"
#include "Filesystem/ext2.h"
#include "paging.h"

#include "Syscalls/syscalls.h"

#define MORE_DEBUG 0

void entry(unsigned long magic, unsigned long addr) {
	multiboot_info_t *mbi;

	if (magic != MULTIBOOT_BOOTLOADER_MAGIC) {
		printf("Invalid magic number 0x%#x\n", magic);
	}

	mbi = (multiboot_info_t*)addr;

	initfb(mbi->framebuffer_info);
	clear();

	if (CHECK_FLAG(mbi->flags, 12)) {

		/* if there is framebuffer information then set that up */

		// load the image and find the pixel array.
		// this is a multiboot module so we don't need a filesystem driver yet
		bmp_t* splash_bmp = (bmp_t*)((module_t*)(mbi->mods_addr))->mod_start;
		uint32_t* splash_array = (uint32_t *)(((module_t*)(mbi->mods_addr))->mod_start + splash_bmp->offset);

		// get the address of the framebuffer
		uint32_t* fb = (uint32_t*) mbi->framebuffer_info.framebuffer_addr_low;

		int row, col;

		// copy the image into the framebuffer
		for (row = 0; row < mbi->framebuffer_info.framebuffer_height; row++) {
			for (col = 0; col < mbi->framebuffer_info.framebuffer_width; col++) {
				uint32_t* pixel = fb + row*mbi->framebuffer_info.framebuffer_width + col;

				*pixel = *(splash_array + (mbi->framebuffer_info.framebuffer_height - row) 
					* mbi->framebuffer_info.framebuffer_width + col);
			}
		}
	}

	printf("flags: 0x%x\n", mbi->flags);

	    /* Construct an LDT entry in the GDT */
    {
        seg_desc_t the_ldt_desc;
        the_ldt_desc.granularity = 0x0;
        the_ldt_desc.opsize      = 0x1;
        the_ldt_desc.reserved    = 0x0;
        the_ldt_desc.avail       = 0x0;
        the_ldt_desc.present     = 0x1;
        the_ldt_desc.dpl         = 0x0;
        the_ldt_desc.sys         = 0x0;
        the_ldt_desc.type        = 0x2;

        SET_LDT_PARAMS(the_ldt_desc, &ldt, ldt_size);
        ldt_desc_ptr = the_ldt_desc;
        lldt(KERNEL_LDT);
    }

    /* Construct a TSS entry in the GDT */
    {
        seg_desc_t the_tss_desc;
        the_tss_desc.granularity   = 0x0;
        the_tss_desc.opsize        = 0x0;
        the_tss_desc.reserved      = 0x0;
        the_tss_desc.avail         = 0x0;
        the_tss_desc.seg_lim_19_16 = TSS_SIZE & 0x000F0000;
        the_tss_desc.present       = 0x1;
        the_tss_desc.dpl           = 0x0;
        the_tss_desc.sys           = 0x0;
        the_tss_desc.type          = 0x9;
        the_tss_desc.seg_lim_15_00 = TSS_SIZE & 0x0000FFFF;

        SET_TSS_PARAMS(the_tss_desc, &tss, tss_size);

        tss_desc_ptr = the_tss_desc;

        tss.ldt_segment_selector = KERNEL_LDT;
        tss.ss0 = KERNEL_DS;
        tss.esp0 = 0x800000;
        ltr(KERNEL_TSS);
    }

	init_idt_exception();

	// we can't use mbi after paging is initialized since its not in kernel memory
	uint32_t fb_addr = mbi->framebuffer_info.framebuffer_addr_low;

	// copy over mmap (can't just do 1 since its a linked list)
	/*
	int i;
	uint8_t mmap[mbi->mmap_length];
	for (i = 0; i < mbi->mmap_length; i++) {
		mmap[i] = *((uint8_t*)mbi->mmap_addr + i);
	}
	uint32_t mmap_length = mbi->mmap_length;
	*/

	init_paging();
	kmap_large(fb_addr, fb_addr);

	i8259_init();
	interrupts_init();
	i8259_enable_irq(1);
	//i8259_enable_irq(12);
	keyboard_init();

	//ata_info(ATA_MASTER);
	//printf("hey mom look at this: %d", 1/0);

	/*
	memory_map_t* newmap = (memory_map_t*)mmap;
	while ((uint32_t)newmap < (uint32_t)mmap + mmap_length) {
		if (newmap->type == 1) {
			printf("Usable memory:\n");
		} else {
			printf("Reserved Memory:\n");
		}
		printf("\tBase Addr: 0x%#x%#x\n", newmap->base_addr_high, newmap->base_addr_low);
		printf("\tLength:    0x%#x%#x\n", newmap->length_high, newmap->length_low);
		
		newmap = (memory_map_t*)((uint32_t)newmap + newmap->size + 4);
	}
	*/

	mbr_t mbr;

	ata_read_sectors(ATA_MASTER, 0, 1, mbr.data);
	ext2_init(mbr.partitions[0].lba_start);

	int32_t error = sys_exec((uint8_t*)"/Programs/bin/shell");
	if (error) {
		printf("Error executing file: %d\n", error);
	}
	

	while (1) {
		asm volatile("hlt");
	}

	return;
}
