/* multiboot.h - Defines used in working with Multiboot-compliant
 * bootloaders (such as GRUB)
 */

#ifndef _MULTIBOOT_H
#define _MULTIBOOT_H

#define MULTIBOOT_HEADER_FLAGS		0x00000007
#define MULTIBOOT_HEADER_MAGIC		0x1BADB002
#define MULTIBOOT_BOOTLOADER_MAGIC	0x2BADB002
#define MULTIBOOT_VIDEO_MODE_TYPE	0
#define MULTIBOOT_VIDEO_WIDTH		1280
#define MULTIBOOT_VIDEO_HEIGHT		720
#define MULTIBOOT_VIDEO_DEPTH		32

#ifndef ASM

/* Types */
#include "types.h"

/* The Multiboot header. */
typedef struct multiboot_header {
	uint32_t magic;
	uint32_t flags;
	uint32_t checksum;
	uint32_t header_addr;
	uint32_t load_addr;
	uint32_t load_end_addr;
	uint32_t bss_end_addr;
	uint32_t entry_addr;
	uint32_t mode_type;
	uint32_t width;
	uint32_t height;
	uint32_t depth;
} multiboot_header_t;

/* The section header table for ELF. */
typedef struct elf_section_header_table {
	uint32_t num;
	uint32_t size;
	uint32_t addr;
	uint32_t shndx;
} elf_section_header_table_t;

typedef struct vbe_info {
	uint32_t vbe_control_info;
	uint32_t vbe_mode_info;
	uint16_t vbe_mode;
	uint16_t vbe_interface_seg;
	uint16_t vbe_interface_off;
	uint16_t vbe_interface_len;
} vbe_info_t;

typedef struct color_info {
	union {
		struct {
			uint32_t framebuffer_palette_addr;
			uint32_t framebuffer_palette_num_colors;
		};

		struct {
			uint8_t framebuffer_red_field_position;
			uint8_t framebuffer_red_mask_size;
			uint8_t framebuffer_green_field_position;
			uint8_t framebuffer_green_mask_size;
			uint8_t framebuffer_blue_field_position;
			uint8_t framebuffer_blue_mask_size;
		};
	};
} color_info_t;

typedef struct color_descriptor {
	uint32_t red_value;
	uint32_t green_value;
	uint32_t blue_value;
} color_descriptor_t;

typedef struct framebuffer_info {
	uint32_t framebuffer_addr_low;
	uint32_t framebuffer_addr_high;
	uint32_t framebuffer_pitch;
	uint32_t framebuffer_width;
	uint32_t framebuffer_height;
	uint8_t framebuffer_bpp;
	uint8_t framebuffer_type;
	color_info_t color_info;
} framebuffer_info_t;

/* The Multiboot information. */
typedef struct multiboot_info {
	uint32_t flags;
	uint32_t mem_lower;
	uint32_t mem_upper;
	uint32_t boot_device;
	uint32_t cmdline;
	uint32_t mods_count;
	uint32_t mods_addr;
	elf_section_header_table_t elf_sec;
	uint32_t mmap_length;
	uint32_t mmap_addr;
	uint32_t drives_length;
	uint32_t drives_addr;
	uint32_t config_table;
	uint32_t boot_loader_name;
	uint32_t apm_table;
	vbe_info_t vbe_info;
	framebuffer_info_t framebuffer_info;
} multiboot_info_t;

typedef struct module {
	uint32_t mod_start;
	uint32_t mod_end;
	uint32_t string;
	uint32_t reserved;
} module_t;

typedef struct memory_map {
	uint32_t size;
	uint32_t base_addr_low;
	uint32_t base_addr_high;
	uint32_t length_low;
	uint32_t length_high;
	uint32_t type;
} memory_map_t;

typedef struct drive {
	uint32_t size;
	uint8_t drive_number;
	uint8_t drive_mode;
	uint16_t drive_cylinders;
	uint8_t drive_heads;
	uint8_t drive_sectors;
	uint16_t drive_ports[];
} drive_t;

typedef struct apm_table {
	uint16_t version;
	uint16_t cseg;
	uint32_t ofset;
	uint16_t cseg_16;
	uint16_t desg;
	uint16_t flags;
	uint16_t cseg_len;
	uint16_t cseg_16_len;
	uint16_t desg_len;
} apm_table_t;

#endif /* ASM */

#endif /* _MULTIBOOT_H */

